@@ -42,11 +42,6 @@ def _is_parent_of(parent, child):
     return False
 
 
-def _get_parent(device):
-    """Return the container device or ``None``."""
-    return device.partition_slave or device.luks_cleartext_slave
-
-
 class Mounter:
 
     """
@@ -80,9 +75,13 @@ class Mounter:
             IgnoreDevice({'symlinks': '/dev/mapper/docker-*', 'ignore': True}),
             IgnoreDevice({'symlinks': '/dev/disk/by-id/dm-name-docker-*',
                           'ignore': True}),
-            IgnoreDevice({'is_loop': True, 'loop_file': '/*', 'ignore': False}),
+            IgnoreDevice({'loop_file': '/var/lib/snapd/snaps/*',
+                          'ignore': True}),
+            IgnoreDevice({'is_loop': True, 'is_ignored': False,
+                          'loop_file': '/*', 'ignore': False}),
             IgnoreDevice({'is_block': False, 'ignore': True}),
-            IgnoreDevice({'is_external': False, 'ignore': True}),
+            IgnoreDevice({'is_external': False,
+                          'is_toplevel': True, 'ignore': True}),
             IgnoreDevice({'is_ignored': True, 'ignore': True})]
         self._prompt = prompt
         self._browser = browser
@@ -532,13 +531,16 @@ class Mounter:
             self._log.warn(_('not detaching {0}: unhandled device', device))
             return False
         drive = device.root
-        if not drive.is_detachable:
+        if not drive.is_detachable and not drive.is_loop:
             self._log.warn(_('not detaching {0}: drive not detachable', drive))
             return False
         if force:
             await self.auto_remove(drive, force=True)
         self._log.debug(_('detaching {0}', device))
-        await drive.detach()
+        if drive.is_detachable:
+            await drive.detach()
+        else:
+            await drive.delete()
         self._log.info(_('detached {0}', device))
         return True
 
@@ -573,7 +575,7 @@ class Mounter:
         return success
 
     # loop devices
-    async def losetup(self, image, read_only=True, offset=None, size=None,
+    async def losetup(self, image, read_only=None, offset=None, size=None,
                       no_part_scan=None):
         """
         Setup a loop device.
@@ -595,12 +597,34 @@ class Mounter:
         if not os.path.isfile(image):
             self._log.error(_('not setting up {0}: not a file', image))
             return None
-        self._log.debug(_('setting up {0}', image))
-        fd = os.open(image, os.O_RDONLY)
+        self._log.debug(_('setting up loop device {0}', image))
+
+        # Create pseudo Device object to enable config matching:
+        loopfile = self.udisks.loopfile_device(image)
+        options = match_config(self._config, loopfile, 'options', [])
+
+        # TODO: also set 'offset', 'size', 'no_part_scan' from config
+        if read_only is None:
+            if 'ro' in options:
+                read_only = True
+            if 'rw' in options:
+                read_only = False
+
+        if not read_only:
+            try:
+                fd = os.open(image, os.O_RDWR)
+            except PermissionError:
+                self._log.debug(_(
+                    'Insufficient permission to open {0} in read-write mode. '
+                    'Retrying in read-only mode.', image))
+                read_only = True
+        if read_only:
+            fd = os.open(image, os.O_RDONLY)
+
         device = await self.udisks.loop_setup(fd, {
             'offset': offset,
             'size': size,
-            'read-only': read_only,
+            'read-only': bool(read_only),
             'no-part-scan': no_part_scan,
         })
         self._log.info(_('set up {0} as {1}', image,
@@ -629,7 +653,7 @@ class Mounter:
 
     # iterate devices
     def is_handleable(self, device):
-        # TODO: handle pathes in first argument
+        # TODO: handle paths in first argument
         """
         Check whether this device should be handled by udiskie.
 
@@ -639,11 +663,7 @@ class Mounter:
         Currently this just means that the device is removable and holds a
         filesystem or the device is a LUKS encrypted volume.
         """
-        ignored = self._ignore_device(device)
-        # propagate handleability of parent devices:
-        if ignored is None and device is not None:
-            return self.is_handleable(_get_parent(device))
-        return not ignored
+        return not self._ignore_device(device)
 
     def is_automount(self, device, default=True):
         if not self.is_handleable(device):
